@@ -1,4 +1,4 @@
-# Copyright (c) 2014, 2021, Oracle and/or its affiliates.
+# Copyright (c) 2014, 2022, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -26,60 +26,76 @@
 # along with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 
-"""Connection class using the C Extension
-"""
-
-# Detection of abstract methods in pylint is not working correctly
-#pylint: disable=W0223
+"""Connection class using the C Extension."""
 
 import os
+import platform
 import socket
-import sysconfig
 
-from . import errors, version
-from .constants import (
-    CharacterSet, FieldFlag, ServerFlag, ShutdownType, ClientFlag
+from . import version
+from .abstracts import MySQLConnectionAbstract
+from .constants import CharacterSet, ClientFlag, FieldFlag, ServerFlag, ShutdownType
+from .errors import (
+    InterfaceError,
+    InternalError,
+    OperationalError,
+    ProgrammingError,
+    get_mysql_exception,
 )
-from .abstracts import MySQLConnectionAbstract, MySQLCursorAbstract
 from .protocol import MySQLProtocol
 
 HAVE_CMYSQL = False
-# pylint: disable=F0401,C0413
+
 try:
     import _mysql_connector
+
+    from _mysql_connector import MySQLInterfaceError
+
     from .cursor_cext import (
-        CMySQLCursor, CMySQLCursorRaw,
-        CMySQLCursorBuffered, CMySQLCursorBufferedRaw, CMySQLCursorPrepared,
-        CMySQLCursorDict, CMySQLCursorBufferedDict, CMySQLCursorNamedTuple,
-        CMySQLCursorBufferedNamedTuple)
-    from _mysql_connector import MySQLInterfaceError  # pylint: disable=F0401
+        CMySQLCursor,
+        CMySQLCursorBuffered,
+        CMySQLCursorBufferedDict,
+        CMySQLCursorBufferedNamedTuple,
+        CMySQLCursorBufferedRaw,
+        CMySQLCursorDict,
+        CMySQLCursorNamedTuple,
+        CMySQLCursorPrepared,
+        CMySQLCursorRaw,
+    )
 except ImportError as exc:
     raise ImportError(
-        "MySQL Connector/Python C Extension not available ({0})".format(
-            str(exc)
-        ))
+        f"MySQL Connector/Python C Extension not available ({exc})"
+    ) from exc
 else:
     HAVE_CMYSQL = True
-# pylint: enable=F0401,C0413
 
 
 class CMySQLConnection(MySQLConnectionAbstract):
-
-    """Class initiating a MySQL Connection using Connector/C"""
+    """Class initiating a MySQL Connection using Connector/C."""
 
     def __init__(self, **kwargs):
         """Initialization"""
         if not HAVE_CMYSQL:
-            raise RuntimeError(
-                "MySQL Connector/Python C Extension not available")
+            raise RuntimeError("MySQL Connector/Python C Extension not available")
         self._cmysql = None
         self._columns = []
         self._plugin_dir = os.path.join(
             os.path.dirname(os.path.abspath(_mysql_connector.__file__)),
-            "mysql", "vendor", "plugin"
+            "mysql",
+            "vendor",
+            "plugin",
         )
+        if platform.system() == "Linux":
+            # Use the authentication plugins from system if they aren't bundled
+            if not os.path.exists(self._plugin_dir):
+                self._plugin_dir = (
+                    "/usr/lib64/mysql/plugin"
+                    if os.path.exists("/usr/lib64/mysql/plugin")
+                    else "/usr/lib/mysql/plugin"
+                )
+
         self.converter = None
-        super(CMySQLConnection, self).__init__(**kwargs)
+        super().__init__()
 
         if kwargs:
             self.connect(**kwargs)
@@ -92,30 +108,32 @@ class CMySQLConnection(MySQLConnectionAbstract):
         else:
             client_license = "Commercial"
 
-        self._conn_attrs.update({
-            "_connector_name": "mysql-connector-python",
-            "_connector_license": client_license,
-            "_connector_version": ".".join(
-                [str(x) for x in version.VERSION[0:3]]),
-            "_source_host": socket.gethostname()
-            })
+        self._conn_attrs.update(
+            {
+                "_connector_name": "mysql-connector-python",
+                "_connector_license": client_license,
+                "_connector_version": ".".join([str(x) for x in version.VERSION[0:3]]),
+                "_source_host": socket.gethostname(),
+            }
+        )
 
     def _do_handshake(self):
         """Gather information of the MySQL server before authentication"""
         self._handshake = {
-            'protocol': self._cmysql.get_proto_info(),
-            'server_version_original': self._cmysql.get_server_info(),
-            'server_threadid': self._cmysql.thread_id(),
-            'charset': None,
-            'server_status': None,
-            'auth_plugin': None,
-            'auth_data': None,
-            'capabilities': self._cmysql.st_server_capabilities(),
+            "protocol": self._cmysql.get_proto_info(),
+            "server_version_original": self._cmysql.get_server_info(),
+            "server_threadid": self._cmysql.thread_id(),
+            "charset": None,
+            "server_status": None,
+            "auth_plugin": None,
+            "auth_data": None,
+            "capabilities": self._cmysql.st_server_capabilities(),
         }
 
         self._server_version = self._check_server_version(
-            self._handshake['server_version_original']
+            self._handshake["server_version_original"]
         )
+        CharacterSet.set_mysql_version(self._server_version)
 
     @property
     def _server_status(self):
@@ -147,17 +165,18 @@ class CMySQLConnection(MySQLConnectionAbstract):
     def autocommit(self):
         """Get whether autocommit is on or off"""
         value = self.info_query("SELECT @@session.autocommit")[0]
-        return True if value == 1 else False
+        return value == 1
 
     @autocommit.setter
-    def autocommit(self, value):  # pylint: disable=W0221
+    def autocommit(self, value):
         """Toggle autocommit"""
         try:
             self._cmysql.autocommit(value)
             self._autocommit = value
-        except MySQLInterfaceError as exc:
-            raise errors.get_mysql_exception(msg=exc.msg, errno=exc.errno,
-                                             sqlstate=exc.sqlstate)
+        except MySQLInterfaceError as err:
+            raise get_mysql_exception(
+                msg=err.msg, errno=err.errno, sqlstate=err.sqlstate
+            ) from err
 
     @property
     def database(self):
@@ -165,7 +184,7 @@ class CMySQLConnection(MySQLConnectionAbstract):
         return self.info_query("SELECT DATABASE()")[0]
 
     @database.setter
-    def database(self, value):  # pylint: disable=W0221
+    def database(self, value):
         """Set the current database"""
         self._cmysql.select_db(value)
 
@@ -176,70 +195,79 @@ class CMySQLConnection(MySQLConnectionAbstract):
 
     def _open_connection(self):
         charset_name = CharacterSet.get_info(self._charset_id)[0]
-        self._cmysql = _mysql_connector.MySQL(  # pylint: disable=E1101,I1101
+        # pylint: disable=c-extension-no-member
+        self._cmysql = _mysql_connector.MySQL(
             buffered=self._buffered,
             raw=self._raw,
             charset_name=charset_name,
             connection_timeout=(self._connection_timeout or 0),
             use_unicode=self._use_unicode,
             auth_plugin=self._auth_plugin,
-            plugin_dir=self._plugin_dir)
+            plugin_dir=self._plugin_dir,
+        )
+        # pylint: enable=c-extension-no-member
         if not self.isset_client_flag(ClientFlag.CONNECT_ARGS):
             self._conn_attrs = {}
         cnx_kwargs = {
-            'host': self._host,
-            'user': self._user,
-            'password': self._password,
-            'password1': self._password1,
-            'password2': self._password2,
-            'password3': self._password3,
-            'database': self._database,
-            'port': self._port,
-            'client_flags': self._client_flags,
-            'unix_socket': self._unix_socket,
-            'compress': self.isset_client_flag(ClientFlag.COMPRESS),
-            'ssl_disabled': True,
+            "host": self._host,
+            "user": self._user,
+            "password": self._password,
+            "password1": self._password1,
+            "password2": self._password2,
+            "password3": self._password3,
+            "database": self._database,
+            "port": self._port,
+            "client_flags": self._client_flags,
+            "unix_socket": self._unix_socket,
+            "compress": self.isset_client_flag(ClientFlag.COMPRESS),
+            "ssl_disabled": True,
             "conn_attrs": self._conn_attrs,
             "local_infile": self._allow_local_infile,
             "load_data_local_dir": self._allow_local_infile_in_path,
             "oci_config_file": self._oci_config_file,
+            "fido_callback": self._fido_callback,
         }
 
-        tls_versions = self._ssl.get('tls_versions')
+        tls_versions = self._ssl.get("tls_versions")
         if tls_versions is not None:
             tls_versions.sort(reverse=True)
             tls_versions = ",".join(tls_versions)
-        if self._ssl.get('tls_ciphersuites') is not None:
-            ssl_ciphersuites = self._ssl.get('tls_ciphersuites')[0]
-            tls_ciphersuites = self._ssl.get('tls_ciphersuites')[1]
+        if self._ssl.get("tls_ciphersuites") is not None:
+            ssl_ciphersuites = self._ssl.get("tls_ciphersuites")[0]
+            tls_ciphersuites = self._ssl.get("tls_ciphersuites")[1]
         else:
             ssl_ciphersuites = None
             tls_ciphersuites = None
-        if tls_versions is not None and "TLSv1.3" in tls_versions and \
-           not tls_ciphersuites:
+        if (
+            tls_versions is not None
+            and "TLSv1.3" in tls_versions
+            and not tls_ciphersuites
+        ):
             tls_ciphersuites = "TLS_AES_256_GCM_SHA384"
         if not self._ssl_disabled:
-            cnx_kwargs.update({
-                'ssl_ca': self._ssl.get('ca'),
-                'ssl_cert': self._ssl.get('cert'),
-                'ssl_key': self._ssl.get('key'),
-                'ssl_cipher_suites': ssl_ciphersuites,
-                'tls_versions': tls_versions,
-                'tls_cipher_suites': tls_ciphersuites,
-                'ssl_verify_cert': self._ssl.get('verify_cert') or False,
-                'ssl_verify_identity':
-                    self._ssl.get('verify_identity') or False,
-                'ssl_disabled': self._ssl_disabled
-            })
+            cnx_kwargs.update(
+                {
+                    "ssl_ca": self._ssl.get("ca"),
+                    "ssl_cert": self._ssl.get("cert"),
+                    "ssl_key": self._ssl.get("key"),
+                    "ssl_cipher_suites": ssl_ciphersuites,
+                    "tls_versions": tls_versions,
+                    "tls_cipher_suites": tls_ciphersuites,
+                    "ssl_verify_cert": self._ssl.get("verify_cert") or False,
+                    "ssl_verify_identity": self._ssl.get("verify_identity") or False,
+                    "ssl_disabled": self._ssl_disabled,
+                }
+            )
 
         try:
             self._cmysql.connect(**cnx_kwargs)
             self._cmysql.converter_str_fallback = self._converter_str_fallback
             if self.converter:
                 self.converter.str_fallback = self._converter_str_fallback
-        except MySQLInterfaceError as exc:
-            raise errors.get_mysql_exception(msg=exc.msg, errno=exc.errno,
-                                             sqlstate=exc.sqlstate)
+        except MySQLInterfaceError as err:
+            raise get_mysql_exception(
+                msg=err.msg, errno=err.errno, sqlstate=err.sqlstate
+            ) from err
 
         self._do_handshake()
 
@@ -249,9 +277,11 @@ class CMySQLConnection(MySQLConnectionAbstract):
             try:
                 self.free_result()
                 self._cmysql.close()
-            except MySQLInterfaceError as exc:
-                raise errors.get_mysql_exception(msg=exc.msg, errno=exc.errno,
-                                                 sqlstate=exc.sqlstate)
+            except MySQLInterfaceError as err:
+                raise get_mysql_exception(
+                    msg=err.msg, errno=err.errno, sqlstate=err.sqlstate
+                ) from err
+
     disconnect = close
 
     def is_closed(self):
@@ -261,6 +291,7 @@ class CMySQLConnection(MySQLConnectionAbstract):
     def is_connected(self):
         """Reports whether the connection to MySQL Server is available"""
         if self._cmysql:
+            self.handle_unread_result()
             return self._cmysql.ping()
 
         return False
@@ -279,7 +310,7 @@ class CMySQLConnection(MySQLConnectionAbstract):
 
         Raises InterfaceError on errors.
         """
-        errmsg = "Connection to MySQL is not available"
+        self.handle_unread_result()
 
         try:
             connected = self._cmysql.ping()
@@ -292,11 +323,10 @@ class CMySQLConnection(MySQLConnectionAbstract):
         if reconnect:
             self.reconnect(attempts=attempts, delay=delay)
         else:
-            raise errors.InterfaceError(errmsg)
+            raise InterfaceError("Connection to MySQL is not available")
 
     def set_character_set_name(self, charset):
-        """Sets the default character set name for current connection.
-        """
+        """Sets the default character set name for current connection."""
         self._cmysql.set_character_set(charset)
 
     def info_query(self, query):
@@ -307,8 +337,7 @@ class CMySQLConnection(MySQLConnectionAbstract):
             first_row = self._cmysql.fetch_row()
             if self._cmysql.fetch_row():
                 self._cmysql.free_result()
-                raise errors.InterfaceError(
-                    "Query should not return more than 1 row")
+                raise InterfaceError("Query should not return more than 1 row")
         self._cmysql.free_result()
 
         return first_row
@@ -323,13 +352,13 @@ class CMySQLConnection(MySQLConnectionAbstract):
 
         return None
 
-    def get_rows(self, count=None, binary=False, columns=None, raw=None,
-                 prep_stmt=None):
+    def get_rows(
+        self, count=None, binary=False, columns=None, raw=None, prep_stmt=None
+    ):
         """Get all or a subset of rows returned by the MySQL server"""
-        unread_result = prep_stmt.have_result_set if prep_stmt \
-            else self.unread_result
+        unread_result = prep_stmt.have_result_set if prep_stmt else self.unread_result
         if not (self._cmysql and unread_result):
-            raise errors.InternalError("No result set available")
+            raise InternalError("No result set available")
 
         if raw is None:
             raw = self._raw
@@ -340,10 +369,7 @@ class CMySQLConnection(MySQLConnectionAbstract):
 
         counter = 0
         try:
-            fetch_row = (
-                prep_stmt.fetch_row if prep_stmt
-                else self._cmysql.fetch_row
-            )
+            fetch_row = prep_stmt.fetch_row if prep_stmt else self._cmysql.fetch_row
             if self.converter:
                 # When using a converter class, the C extension should not
                 # convert the values. This can be accomplished by setting
@@ -355,8 +381,7 @@ class CMySQLConnection(MySQLConnectionAbstract):
                     row = list(row)
                     for i, _ in enumerate(row):
                         if not raw:
-                            row[i] = self.converter.to_python(self._columns[i],
-                                                              row[i])
+                            row[i] = self.converter.to_python(self._columns[i], row[i])
                     row = tuple(row)
                 rows.append(row)
                 counter += 1
@@ -364,7 +389,7 @@ class CMySQLConnection(MySQLConnectionAbstract):
                     break
                 row = fetch_row()
             if not row:
-                _eof = self.fetch_eof_columns(prep_stmt)['eof']
+                _eof = self.fetch_eof_columns(prep_stmt)["eof"]
                 if prep_stmt:
                     prep_stmt.free_result()
                     self._unread_result = False
@@ -372,22 +397,27 @@ class CMySQLConnection(MySQLConnectionAbstract):
                     self.free_result()
             else:
                 _eof = None
-        except MySQLInterfaceError as exc:
+        except MySQLInterfaceError as err:
             if prep_stmt:
                 prep_stmt.free_result()
-                raise errors.InterfaceError(str(exc))
-            else:
-                self.free_result()
-                raise errors.get_mysql_exception(msg=exc.msg, errno=exc.errno,
-                                                 sqlstate=exc.sqlstate)
+                raise InterfaceError(str(err)) from err
+            self.free_result()
+            raise get_mysql_exception(
+                msg=err.msg, errno=err.errno, sqlstate=err.sqlstate
+            ) from err
 
         return rows, _eof
 
     def get_row(self, binary=False, columns=None, raw=None, prep_stmt=None):
         """Get the next rows returned by the MySQL server"""
         try:
-            rows, eof = self.get_rows(count=1, binary=binary, columns=columns,
-                                      raw=raw, prep_stmt=prep_stmt)
+            rows, eof = self.get_rows(
+                count=1,
+                binary=binary,
+                columns=columns,
+                raw=raw,
+                prep_stmt=prep_stmt,
+            )
             if rows:
                 return (rows[0], eof)
             return (None, eof)
@@ -410,6 +440,7 @@ class CMySQLConnection(MySQLConnectionAbstract):
     def commit(self):
         """Commit current transaction"""
         if self._cmysql:
+            self.handle_unread_result()
             self._cmysql.commit()
 
     def rollback(self):
@@ -422,50 +453,53 @@ class CMySQLConnection(MySQLConnectionAbstract):
         """Change the current database"""
         try:
             self._cmysql.select_db(database)
-        except MySQLInterfaceError as exc:
-            raise errors.get_mysql_exception(msg=exc.msg, errno=exc.errno,
-                                             sqlstate=exc.sqlstate)
+        except MySQLInterfaceError as err:
+            raise get_mysql_exception(
+                msg=err.msg, errno=err.errno, sqlstate=err.sqlstate
+            ) from err
 
     def fetch_eof_columns(self, prep_stmt=None):
         """Fetch EOF and column information"""
-        have_result_set = prep_stmt.have_result_set if prep_stmt \
-            else self._cmysql.have_result_set
+        have_result_set = (
+            prep_stmt.have_result_set if prep_stmt else self._cmysql.have_result_set
+        )
         if not have_result_set:
-            raise errors.InterfaceError("No result set")
+            raise InterfaceError("No result set")
 
-        fields = prep_stmt.fetch_fields() if prep_stmt \
-            else self._cmysql.fetch_fields()
+        fields = prep_stmt.fetch_fields() if prep_stmt else self._cmysql.fetch_fields()
         self._columns = []
         for col in fields:
-            self._columns.append((
-                col[4],
-                int(col[8]),
-                None,
-                None,
-                None,
-                None,
-                ~int(col[9]) & FieldFlag.NOT_NULL,
-                int(col[9]),
-                int(col[6]),
-            ))
+            self._columns.append(
+                (
+                    col[4],
+                    int(col[8]),
+                    None,
+                    None,
+                    None,
+                    None,
+                    ~int(col[9]) & FieldFlag.NOT_NULL,
+                    int(col[9]),
+                    int(col[6]),
+                )
+            )
 
         return {
-            'eof': {
-                'status_flag': self._server_status,
-                'warning_count': self._cmysql.st_warning_count(),
+            "eof": {
+                "status_flag": self._server_status,
+                "warning_count": self._cmysql.st_warning_count(),
             },
-            'columns': self._columns,
+            "columns": self._columns,
         }
 
     def fetch_eof_status(self):
         """Fetch EOF and status information"""
         if self._cmysql:
             return {
-                'warning_count': self._cmysql.st_warning_count(),
-                'field_count': self._cmysql.st_field_count(),
-                'insert_id': self._cmysql.insert_id(),
-                'affected_rows': self._cmysql.affected_rows(),
-                'server_status': self._server_status,
+                "warning_count": self._cmysql.st_warning_count(),
+                "field_count": self._cmysql.st_field_count(),
+                "insert_id": self._cmysql.insert_id(),
+                "affected_rows": self._cmysql.affected_rows(),
+                "server_status": self._server_status,
             }
 
         return None
@@ -473,44 +507,42 @@ class CMySQLConnection(MySQLConnectionAbstract):
     def cmd_stmt_prepare(self, statement):
         """Prepares the SQL statement"""
         if not self._cmysql:
-            raise errors.OperationalError("MySQL Connection not available")
+            raise OperationalError("MySQL Connection not available")
 
         try:
             stmt = self._cmysql.stmt_prepare(statement)
             stmt.converter_str_fallback = self._converter_str_fallback
             return stmt
         except MySQLInterfaceError as err:
-            raise errors.InterfaceError(str(err))
+            raise InterfaceError(str(err)) from err
 
-    # pylint: disable=W0221
-    def cmd_stmt_execute(self, prep_stmt, *args):
+    def cmd_stmt_execute(self, statement_id, *args):
         """Executes the prepared statement"""
         try:
-            prep_stmt.stmt_execute(*args)
+            statement_id.stmt_execute(*args)
         except MySQLInterfaceError as err:
-            raise errors.InterfaceError(str(err))
+            raise InterfaceError(str(err)) from err
 
         self._columns = []
-        if not prep_stmt.have_result_set:
+        if not statement_id.have_result_set:
             # No result
             self._unread_result = False
             return self.fetch_eof_status()
 
         self._unread_result = True
-        return self.fetch_eof_columns(prep_stmt)
+        return self.fetch_eof_columns(statement_id)
 
-    def cmd_stmt_close(self, prep_stmt):
+    def cmd_stmt_close(self, statement_id):
         """Closes the prepared statement"""
         if self._unread_result:
-            raise errors.InternalError("Unread result found")
-        prep_stmt.stmt_close()
+            raise InternalError("Unread result found")
+        statement_id.stmt_close()
 
-    def cmd_stmt_reset(self, prep_stmt):
+    def cmd_stmt_reset(self, statement_id):
         """Resets the prepared statement"""
         if self._unread_result:
-            raise errors.InternalError("Unread result found")
-        prep_stmt.stmt_reset()
-    # pylint: enable=W0221
+            raise InternalError("Unread result found")
+        statement_id.stmt_reset()
 
     def cmd_query(self, query, raw=None, buffered=False, raw_as_string=False):
         """Send a query to the MySQL server"""
@@ -519,21 +551,25 @@ class CMySQLConnection(MySQLConnectionAbstract):
             raw = self._raw
         try:
             if not isinstance(query, bytes):
-                query = query.encode('utf-8')
-            self._cmysql.query(query,
-                               raw=raw, buffered=buffered,
-                               raw_as_string=raw_as_string,
-                               query_attrs=self._query_attrs)
-        except MySQLInterfaceError as exc:
-            raise errors.get_mysql_exception(exc.errno, msg=exc.msg,
-                                             sqlstate=exc.sqlstate)
-        except AttributeError:
-            if self._unix_socket:
-                addr = self._unix_socket
-            else:
-                addr = self._host + ':' + str(self._port)
-            raise errors.OperationalError(
-                errno=2055, values=(addr, 'Connection not available.'))
+                query = query.encode("utf-8")
+            self._cmysql.query(
+                query,
+                raw=raw,
+                buffered=buffered,
+                raw_as_string=raw_as_string,
+                query_attrs=self._query_attrs,
+            )
+        except MySQLInterfaceError as err:
+            raise get_mysql_exception(
+                err.errno, msg=err.msg, sqlstate=err.sqlstate
+            ) from err
+        except AttributeError as err:
+            addr = (
+                self._unix_socket if self._unix_socket else f"{self._host}:{self._port}"
+            )
+            raise OperationalError(
+                errno=2055, values=(addr, "Connection not available.")
+            ) from err
 
         self._columns = []
         if not self._cmysql.have_result_set:
@@ -541,10 +577,18 @@ class CMySQLConnection(MySQLConnectionAbstract):
             return self.fetch_eof_status()
 
         return self.fetch_eof_columns()
+
     _execute_query = cmd_query
 
-    def cursor(self, buffered=None, raw=None, prepared=None, cursor_class=None,
-               dictionary=None, named_tuple=None):
+    def cursor(
+        self,
+        buffered=None,
+        raw=None,
+        prepared=None,
+        cursor_class=None,
+        dictionary=None,
+        named_tuple=None,
+    ):
         """Instantiates and returns a cursor using C Extension
 
         By default, CMySQLCursor is returned. Depending on the options
@@ -575,12 +619,12 @@ class CMySQLConnection(MySQLConnectionAbstract):
         """
         self.handle_unread_result(prepared)
         if not self.is_connected():
-            raise errors.OperationalError("MySQL Connection not available.")
+            raise OperationalError("MySQL Connection not available.")
         if cursor_class is not None:
-            if not issubclass(cursor_class, MySQLCursorAbstract):
-                raise errors.ProgrammingError(
-                    "Cursor class needs be to subclass"
-                    " of cursor_cext.CMySQLCursor")
+            if not issubclass(cursor_class, CMySQLCursor):
+                raise ProgrammingError(
+                    "Cursor class needs be to subclass of cursor_cext.CMySQLCursor"
+                )
             return (cursor_class)(self)
 
         buffered = buffered or self._buffered
@@ -607,21 +651,22 @@ class CMySQLConnection(MySQLConnectionAbstract):
             5: CMySQLCursorBufferedDict,
             8: CMySQLCursorNamedTuple,
             9: CMySQLCursorBufferedNamedTuple,
-            16: CMySQLCursorPrepared
+            16: CMySQLCursorPrepared,
         }
         try:
             return (types[cursor_type])(self)
         except KeyError:
-            args = ('buffered', 'raw', 'dictionary', 'named_tuple', 'prepared')
-            raise ValueError('Cursor not available with given criteria: ' +
-                             ', '.join([args[i] for i in range(5)
-                                        if cursor_type & (1 << i) != 0]))
+            args = ("buffered", "raw", "dictionary", "named_tuple", "prepared")
+            raise ValueError(
+                "Cursor not available with given criteria: "
+                + ", ".join([args[i] for i in range(5) if cursor_type & (1 << i) != 0])
+            ) from None
 
     @property
     def num_rows(self):
         """Returns number of rows of current result set"""
         if not self._cmysql.have_result_set:
-            raise errors.InterfaceError("No result set")
+            raise InterfaceError("No result set")
 
         return self._cmysql.num_rows()
 
@@ -663,10 +708,9 @@ class CMySQLConnection(MySQLConnectionAbstract):
             if self.converter:
                 result = [
                     self.converter.quote(
-                        self.converter.escape(
-                            self.converter.to_mysql(value)
-                        )
-                    ) for value in params
+                        self.converter.escape(self.converter.to_mysql(value))
+                    )
+                    for value in params
                 ]
             else:
                 result = self._cmysql.convert_to_mysql(*params)
@@ -675,17 +719,16 @@ class CMySQLConnection(MySQLConnectionAbstract):
             if self.converter:
                 for key, value in params.items():
                     result[key] = self.converter.quote(
-                        self.converter.escape(
-                            self.converter.to_mysql(value)
-                        )
+                        self.converter.escape(self.converter.to_mysql(value))
                     )
             else:
                 for key, value in params.items():
                     result[key] = self._cmysql.convert_to_mysql(value)[0]
         else:
-            raise errors.ProgrammingError(
+            raise ProgrammingError(
                 f"Could not process parameters: {type(params).__name__}({params}),"
-                " it must be of type list, tuple or dict")
+                " it must be of type list, tuple or dict"
+            )
 
         return result
 
@@ -696,9 +739,17 @@ class CMySQLConnection(MySQLConnectionAbstract):
         """
         self._cmysql.consume_result()
 
-    def cmd_change_user(self, username='', password='', database='',
-                        charset=45, password1='', password2='', password3='',
-                        oci_config_file=None):
+    def cmd_change_user(
+        self,
+        username="",
+        password="",
+        database="",
+        charset=45,
+        password1="",
+        password2="",
+        password3="",
+        oci_config_file=None,
+    ):
         """Change the current logged in user"""
         try:
             self._cmysql.change_user(
@@ -708,11 +759,13 @@ class CMySQLConnection(MySQLConnectionAbstract):
                 password1,
                 password2,
                 password3,
-                oci_config_file)
+                oci_config_file,
+            )
 
-        except MySQLInterfaceError as exc:
-            raise errors.get_mysql_exception(msg=exc.msg, errno=exc.errno,
-                                             sqlstate=exc.sqlstate)
+        except MySQLInterfaceError as err:
+            raise get_mysql_exception(
+                msg=err.msg, errno=err.errno, sqlstate=err.sqlstate
+            ) from err
 
         self._charset_id = charset
         self._post_connection()
@@ -720,27 +773,25 @@ class CMySQLConnection(MySQLConnectionAbstract):
     def cmd_reset_connection(self):
         """Resets the session state without re-authenticating
 
-        Works only for MySQL server 5.7.3 or later.
-        """
-        if self._server_version < (5, 7, 3):
-            raise errors.NotSupportedError("MySQL version 5.7.2 and "
-                                           "earlier does not support "
-                                           "COM_RESET_CONNECTION.")
-        try:
-            self._cmysql.reset_connection()
-        except MySQLInterfaceError as exc:
-            raise errors.get_mysql_exception(msg=exc.msg, errno=exc.errno,
-                                             sqlstate=exc.sqlstate)
+        Reset command only works on MySQL server 5.7.3 or later.
+        The result is True for a successful reset otherwise False.
 
-        self._post_connection()
+        Returns bool
+        """
+        res = self._cmysql.reset_connection()
+        if res:
+            self._post_connection()
+        return res
 
     def cmd_refresh(self, options):
         """Send the Refresh command to the MySQL server"""
         try:
+            self.handle_unread_result()
             self._cmysql.refresh(options)
-        except MySQLInterfaceError as exc:
-            raise errors.get_mysql_exception(msg=exc.msg, errno=exc.errno,
-                                             sqlstate=exc.sqlstate)
+        except MySQLInterfaceError as err:
+            raise get_mysql_exception(
+                msg=err.msg, errno=err.errno, sqlstate=err.sqlstate
+            ) from err
 
         return self.fetch_eof_status()
 
@@ -751,20 +802,21 @@ class CMySQLConnection(MySQLConnectionAbstract):
     def cmd_shutdown(self, shutdown_type=None):
         """Shut down the MySQL Server"""
         if not self._cmysql:
-            raise errors.OperationalError("MySQL Connection not available")
+            raise OperationalError("MySQL Connection not available")
 
         if shutdown_type:
             if not ShutdownType.get_info(shutdown_type):
-                raise errors.InterfaceError("Invalid shutdown type")
+                raise InterfaceError("Invalid shutdown type")
             level = shutdown_type
         else:
             level = ShutdownType.SHUTDOWN_DEFAULT
 
         try:
             self._cmysql.shutdown(level)
-        except MySQLInterfaceError as exc:
-            raise errors.get_mysql_exception(msg=exc.msg, errno=exc.errno,
-                                             sqlstate=exc.sqlstate)
+        except MySQLInterfaceError as err:
+            raise get_mysql_exception(
+                msg=err.msg, errno=err.errno, sqlstate=err.sqlstate
+            ) from err
         self.close()
 
     def cmd_statistics(self):
@@ -774,24 +826,40 @@ class CMySQLConnection(MySQLConnectionAbstract):
         try:
             stat = self._cmysql.stat()
             return MySQLProtocol().parse_statistics(stat, with_header=False)
-        except (MySQLInterfaceError, errors.InterfaceError) as exc:
-            raise errors.get_mysql_exception(msg=exc.msg, errno=exc.errno,
-                                             sqlstate=exc.sqlstate)
+        except (MySQLInterfaceError, InterfaceError) as err:
+            raise get_mysql_exception(
+                msg=err.msg, errno=err.errno, sqlstate=err.sqlstate
+            ) from err
 
     def cmd_process_kill(self, mysql_pid):
         """Kill a MySQL process"""
         if not isinstance(mysql_pid, int):
             raise ValueError("MySQL PID must be int")
-        self.info_query("KILL {0}".format(mysql_pid))
+        self.info_query(f"KILL {mysql_pid}")
+
+    def cmd_debug(self):
+        """Send the DEBUG command"""
+        raise NotImplementedError
+
+    def cmd_ping(self):
+        """Send the PING command"""
+        raise NotImplementedError
+
+    def cmd_query_iter(self, statements):
+        """Send one or more statements to the MySQL server"""
+        raise NotImplementedError
+
+    def cmd_stmt_send_long_data(self, statement_id, param_id, data):
+        """Send data for a column"""
+        raise NotImplementedError
 
     def handle_unread_result(self, prepared=False):
         """Check whether there is an unread result"""
-        unread_result = self._unread_result if prepared is True \
-            else self.unread_result
+        unread_result = self._unread_result if prepared is True else self.unread_result
         if self.can_consume_results:
             self.consume_results()
         elif unread_result:
-            raise errors.InternalError("Unread result found")
+            raise InternalError("Unread result found")
 
     def reset_session(self, user_variables=None, session_variables=None):
         """Clears the current active session
@@ -810,31 +878,29 @@ class CMySQLConnection(MySQLConnectionAbstract):
         unread results and InterfaceError on errors.
         """
         if not self.is_connected():
-            raise errors.OperationalError("MySQL Connection not available.")
+            raise OperationalError("MySQL Connection not available.")
 
-        try:
-            self.cmd_reset_connection()
-        except (errors.NotSupportedError, NotImplementedError):
-            if self._compress:
-                raise errors.NotSupportedError(
-                    "Reset session is not supported with compression for "
-                    "MySQL server version 5.7.2 or earlier.")
-            elif self._server_version < (5, 7, 3):
-                raise errors.NotSupportedError(
-                    "Reset session is not supported with MySQL server "
-                    "version 5.7.2 or earlier.")
-            else:
-                self.cmd_change_user(self._user, self._password,
-                                     self._database, self._charset_id,
-                                     self._password1, self._password2,
-                                     self._password3, self._oci_config_file)
+        if not self.cmd_reset_connection():
+            try:
+                self.cmd_change_user(
+                    self._user,
+                    self._password,
+                    self._database,
+                    self._charset_id,
+                    self._password1,
+                    self._password2,
+                    self._password3,
+                    self._oci_config_file,
+                )
+            except ProgrammingError:
+                self.reconnect()
 
         if user_variables or session_variables:
             cur = self.cursor()
             if user_variables:
                 for key, value in user_variables.items():
-                    cur.execute("SET @`{0}` = %s".format(key), (value,))
+                    cur.execute(f"SET @`{key}` = %s", (value,))
             if session_variables:
                 for key, value in session_variables.items():
-                    cur.execute("SET SESSION `{0}` = %s".format(key), (value,))
+                    cur.execute(f"SET SESSION `{key}` = %s", (value,))
             cur.close()
