@@ -1,4 +1,4 @@
-# Copyright (c) 2009, 2020, Oracle and/or its affiliates.
+# Copyright (c) 2009, 2022, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -30,42 +30,46 @@
 """
 
 import datetime
+import math
+import struct
 import time
+
 from decimal import Decimal
 
-from .constants import FieldType, FieldFlag, CharacterSet
-from .catch23 import PY2, NUMERIC_TYPES, struct_unpack
+from .constants import CharacterSet, FieldFlag, FieldType
 from .custom_types import HexLiteral
+from .utils import NUMERIC_TYPES
 
 CONVERT_ERROR = "Could not convert '{value}' to python {pytype}"
 
 
-class MySQLConverterBase(object):
+class MySQLConverterBase:
     """Base class for conversion classes
 
     All class dealing with converting to and from MySQL data types must
     be a subclass of this class.
     """
 
-    def __init__(self, charset='utf8', use_unicode=True):
+    def __init__(self, charset="utf8", use_unicode=True, str_fallback=False):
         self.python_types = None
         self.mysql_types = None
         self.charset = None
         self.charset_id = 0
         self.use_unicode = None
         self.set_charset(charset)
-        self.set_unicode(use_unicode)
+        self.use_unicode = use_unicode
+        self.str_fallback = str_fallback
         self._cache_field_types = {}
 
     def set_charset(self, charset):
         """Set character set"""
-        if charset == 'utf8mb4':
-            charset = 'utf8'
+        if charset in ("utf8mb4", "utf8mb3"):
+            charset = "utf8"
         if charset is not None:
             self.charset = charset
         else:
             # default to utf8
-            self.charset = 'utf8'
+            self.charset = "utf8"
         self.charset_id = CharacterSet.get_charset_info(self.charset)[0]
 
     def set_unicode(self, value=True):
@@ -76,14 +80,14 @@ class MySQLConverterBase(object):
         """Convert Python data type to MySQL"""
         type_name = value.__class__.__name__.lower()
         try:
-            return getattr(self, "_{0}_to_mysql".format(type_name))(value)
+            return getattr(self, f"_{type_name}_to_mysql")(value)
         except AttributeError:
             return value
 
     def to_python(self, vtype, value):
         """Convert MySQL data type to Python"""
 
-        if (value == b'\x00' or value is None) and vtype[1] != FieldType.BIT:
+        if (value == b"\x00" or value is None) and vtype[1] != FieldType.BIT:
             # Don't go further when we hit a NULL value
             return None
 
@@ -92,7 +96,8 @@ class MySQLConverterBase(object):
             for name, info in FieldType.desc.items():
                 try:
                     self._cache_field_types[info[0]] = getattr(
-                        self, '_{0}_to_python'.format(name))
+                        self, f"_{name.lower()}_to_python"
+                    )
                 except AttributeError:
                     # We ignore field types which has no method
                     pass
@@ -102,11 +107,13 @@ class MySQLConverterBase(object):
         except KeyError:
             return value
 
-    def escape(self, value):
+    @staticmethod
+    def escape(value):
         """Escape buffer for sending to MySQL"""
         return value
 
-    def quote(self, buf):
+    @staticmethod
+    def quote(buf):
         """Quote buffer for sending to MySQL"""
         return str(buf)
 
@@ -125,11 +132,12 @@ class MySQLConverter(MySQLConverterBase):
 
     """
 
-    def __init__(self, charset=None, use_unicode=True):
-        MySQLConverterBase.__init__(self, charset, use_unicode)
+    def __init__(self, charset=None, use_unicode=True, str_fallback=False):
+        MySQLConverterBase.__init__(self, charset, use_unicode, str_fallback)
         self._cache_field_types = {}
 
-    def escape(self, value):
+    @staticmethod
+    def escape(value):
         """
         Escapes special characters as they are expected to by when MySQL
         receives them.
@@ -139,25 +147,26 @@ class MySQLConverter(MySQLConverterBase):
         """
         if value is None:
             return value
-        elif isinstance(value, NUMERIC_TYPES):
+        if isinstance(value, NUMERIC_TYPES):
             return value
         if isinstance(value, (bytes, bytearray)):
-            value = value.replace(b'\\', b'\\\\')
-            value = value.replace(b'\n', b'\\n')
-            value = value.replace(b'\r', b'\\r')
-            value = value.replace(b'\047', b'\134\047')  # single quotes
-            value = value.replace(b'\042', b'\134\042')  # double quotes
-            value = value.replace(b'\032', b'\134\032')  # for Win32
+            value = value.replace(b"\\", b"\\\\")
+            value = value.replace(b"\n", b"\\n")
+            value = value.replace(b"\r", b"\\r")
+            value = value.replace(b"\047", b"\134\047")  # single quotes
+            value = value.replace(b"\042", b"\134\042")  # double quotes
+            value = value.replace(b"\032", b"\134\032")  # for Win32
         else:
-            value = value.replace('\\', '\\\\')
-            value = value.replace('\n', '\\n')
-            value = value.replace('\r', '\\r')
-            value = value.replace('\047', '\134\047')  # single quotes
-            value = value.replace('\042', '\134\042')  # double quotes
-            value = value.replace('\032', '\134\032')  # for Win32
+            value = value.replace("\\", "\\\\")
+            value = value.replace("\n", "\\n")
+            value = value.replace("\r", "\\r")
+            value = value.replace("\047", "\134\047")  # single quotes
+            value = value.replace("\042", "\134\042")  # double quotes
+            value = value.replace("\032", "\134\032")  # for Win32
         return value
 
-    def quote(self, buf):
+    @staticmethod
+    def quote(buf):
         """
         Quote the parameters for commands. General rules:
           o numbers are returns as bytes using ascii codec
@@ -167,12 +176,8 @@ class MySQLConverter(MySQLConverterBase):
         Returns a bytearray object.
         """
         if isinstance(buf, NUMERIC_TYPES):
-            if PY2:
-                if isinstance(buf, float):
-                    return repr(buf)
-                return str(buf)
-            return str(buf).encode('ascii')
-        elif isinstance(buf, type(None)):
+            return str(buf).encode("ascii")
+        if isinstance(buf, type(None)):
             return bytearray(b"NULL")
         return bytearray(b"'" + buf + b"'")
 
@@ -180,10 +185,13 @@ class MySQLConverter(MySQLConverterBase):
         """Convert Python data type to MySQL"""
         type_name = value.__class__.__name__.lower()
         try:
-            return getattr(self, "_{0}_to_mysql".format(type_name))(value)
+            return getattr(self, f"_{type_name}_to_mysql")(value)
         except AttributeError:
-            raise TypeError("Python '{0}' cannot be converted to a "
-                            "MySQL type".format(type_name))
+            if self.str_fallback:
+                return str(value).encode()
+            raise TypeError(
+                f"Python '{type_name}' cannot be converted to a MySQL type"
+            ) from None
 
     def to_python(self, vtype, value):
         """Convert MySQL data type to Python"""
@@ -198,7 +206,8 @@ class MySQLConverter(MySQLConverterBase):
             for name, info in FieldType.desc.items():
                 try:
                     self._cache_field_types[info[0]] = getattr(
-                        self, '_{0}_to_python'.format(name))
+                        self, f"_{name.lower()}_to_python"
+                    )
                 except AttributeError:
                     # We ignore field types which has no method
                     pass
@@ -208,62 +217,67 @@ class MySQLConverter(MySQLConverterBase):
         except KeyError:
             # If one type is not defined, we just return the value as str
             try:
-                return value.decode('utf-8')
+                return value.decode("utf-8")
             except UnicodeDecodeError:
                 return value
         except ValueError as err:
-            raise ValueError("%s (field %s)" % (err, vtype[0]))
+            raise ValueError(f"{err} (field {vtype[0]})") from err
         except TypeError as err:
-            raise TypeError("%s (field %s)" % (err, vtype[0]))
-        except:
-            raise
+            raise TypeError(f"{err} (field {vtype[0]})") from err
 
-    def _int_to_mysql(self, value):
+    @staticmethod
+    def _int_to_mysql(value):
         """Convert value to int"""
         return int(value)
 
-    def _long_to_mysql(self, value):
+    @staticmethod
+    def _long_to_mysql(value):
         """Convert value to int"""
         return int(value)
 
-    def _float_to_mysql(self, value):
+    @staticmethod
+    def _float_to_mysql(value):
         """Convert value to float"""
+        if math.isnan(value):
+            return None
         return float(value)
 
     def _str_to_mysql(self, value):
         """Convert value to string"""
-        if PY2:
-            return str(value)
         return self._unicode_to_mysql(value)
 
     def _unicode_to_mysql(self, value):
         """Convert unicode"""
         charset = self.charset
         charset_id = self.charset_id
-        if charset == 'binary':
-            charset = 'utf8'
+        if charset == "binary":
+            charset = "utf8"
             charset_id = CharacterSet.get_charset_info(charset)[0]
         encoded = value.encode(charset)
         if charset_id in CharacterSet.slash_charsets:
-            if b'\x5c' in encoded:
+            if b"\x5c" in encoded:
                 return HexLiteral(value, charset)
         return encoded
 
-    def _bytes_to_mysql(self, value):
+    @staticmethod
+    def _bytes_to_mysql(value):
         """Convert value to bytes"""
         return value
 
-    def _bytearray_to_mysql(self, value):
+    @staticmethod
+    def _bytearray_to_mysql(value):
         """Convert value to bytes"""
         return bytes(value)
 
-    def _bool_to_mysql(self, value):
+    @staticmethod
+    def _bool_to_mysql(value):
         """Convert value to boolean"""
         if value:
             return 1
         return 0
 
-    def _nonetype_to_mysql(self, value):
+    @staticmethod
+    def _nonetype_to_mysql(value):  # pylint: disable=unused-argument
         """
         This would return what None would be in MySQL, but instead we
         leave it None and return it right away. The actual conversion
@@ -273,7 +287,8 @@ class MySQLConverter(MySQLConverterBase):
         """
         return None
 
-    def _datetime_to_mysql(self, value):
+    @staticmethod
+    def _datetime_to_mysql(value):
         """
         Converts a datetime instance to a string suitable for MySQL.
         The returned string has format: %Y-%m-%d %H:%M:%S[.%f]
@@ -283,18 +298,29 @@ class MySQLConverter(MySQLConverterBase):
         Returns a bytes.
         """
         if value.microsecond:
-            fmt = '{0:04d}-{1:02d}-{2:02d} {3:02d}:{4:02d}:{5:02d}.{6:06d}'
+            fmt = "{0:04d}-{1:02d}-{2:02d} {3:02d}:{4:02d}:{5:02d}.{6:06d}"
             return fmt.format(
-                value.year, value.month, value.day,
-                value.hour, value.minute, value.second,
-                value.microsecond).encode('ascii')
+                value.year,
+                value.month,
+                value.day,
+                value.hour,
+                value.minute,
+                value.second,
+                value.microsecond,
+            ).encode("ascii")
 
-        fmt = '{0:04d}-{1:02d}-{2:02d} {3:02d}:{4:02d}:{5:02d}'
+        fmt = "{0:04d}-{1:02d}-{2:02d} {3:02d}:{4:02d}:{5:02d}"
         return fmt.format(
-            value.year, value.month, value.day,
-            value.hour, value.minute, value.second).encode('ascii')
+            value.year,
+            value.month,
+            value.day,
+            value.hour,
+            value.minute,
+            value.second,
+        ).encode("ascii")
 
-    def _date_to_mysql(self, value):
+    @staticmethod
+    def _date_to_mysql(value):
         """
         Converts a date instance to a string suitable for MySQL.
         The returned string has format: %Y-%m-%d
@@ -303,10 +329,10 @@ class MySQLConverter(MySQLConverterBase):
 
         Returns a bytes.
         """
-        return '{0:04d}-{1:02d}-{2:02d}'.format(value.year, value.month,
-                                                value.day).encode('ascii')
+        return f"{value.year:04d}-{value.month:02d}-{value.day:02d}".encode("ascii")
 
-    def _time_to_mysql(self, value):
+    @staticmethod
+    def _time_to_mysql(value):
         """
         Converts a time instance to a string suitable for MySQL.
         The returned string has format: %H:%M:%S[.%f]
@@ -316,10 +342,11 @@ class MySQLConverter(MySQLConverterBase):
         Returns a bytes.
         """
         if value.microsecond:
-            return value.strftime('%H:%M:%S.%f').encode('ascii')
-        return value.strftime('%H:%M:%S').encode('ascii')
+            return value.strftime("%H:%M:%S.%f").encode("ascii")
+        return value.strftime("%H:%M:%S").encode("ascii")
 
-    def _struct_time_to_mysql(self, value):
+    @staticmethod
+    def _struct_time_to_mysql(value):
         """
         Converts a time.struct_time sequence to a string suitable
         for MySQL.
@@ -327,9 +354,10 @@ class MySQLConverter(MySQLConverterBase):
 
         Returns a bytes or None when not valid.
         """
-        return time.strftime('%Y-%m-%d %H:%M:%S', value).encode('ascii')
+        return time.strftime("%Y-%m-%d %H:%M:%S", value).encode("ascii")
 
-    def _timedelta_to_mysql(self, value):
+    @staticmethod
+    def _timedelta_to_mysql(value):
         """
         Converts a timedelta instance to a string suitable for MySQL.
         The returned string has format: %H:%M:%S
@@ -339,17 +367,17 @@ class MySQLConverter(MySQLConverterBase):
         seconds = abs(value.days * 86400 + value.seconds)
 
         if value.microseconds:
-            fmt = '{0:02d}:{1:02d}:{2:02d}.{3:06d}'
+            fmt = "{0:02d}:{1:02d}:{2:02d}.{3:06d}"
             if value.days < 0:
                 mcs = 1000000 - value.microseconds
                 seconds -= 1
             else:
                 mcs = value.microseconds
         else:
-            fmt = '{0:02d}:{1:02d}:{2:02d}'
+            fmt = "{0:02d}:{1:02d}:{2:02d}"
 
         if value.days < 0:
-            fmt = '-' + fmt
+            fmt = "-" + fmt
 
         (hours, remainder) = divmod(seconds, 3600)
         (mins, secs) = divmod(remainder, 60)
@@ -359,11 +387,10 @@ class MySQLConverter(MySQLConverterBase):
         else:
             result = fmt.format(hours, mins, secs)
 
-        if PY2:
-            return result
-        return result.encode('ascii')
+        return result.encode("ascii")
 
-    def _decimal_to_mysql(self, value):
+    @staticmethod
+    def _decimal_to_mysql(value):
         """
         Converts a decimal.Decimal instance to a string suitable for
         MySQL.
@@ -371,7 +398,7 @@ class MySQLConverter(MySQLConverterBase):
         Returns a bytes or None when not valid.
         """
         if isinstance(value, Decimal):
-            return str(value).encode('ascii')
+            return str(value).encode("ascii")
 
         return None
 
@@ -385,14 +412,15 @@ class MySQLConverter(MySQLConverterBase):
         Returns a tuple.
         """
         i = 0
-        result = [None]*len(fields)
+        result = [None] * len(fields)
 
         if not self._cache_field_types:
             self._cache_field_types = {}
             for name, info in FieldType.desc.items():
                 try:
                     self._cache_field_types[info[0]] = getattr(
-                        self, '_{0}_to_python'.format(name))
+                        self, f"_{name.lower()}_to_python"
+                    )
                 except AttributeError:
                     # We ignore field types which has no method
                     pass
@@ -410,60 +438,66 @@ class MySQLConverter(MySQLConverterBase):
             except KeyError:
                 # If one type is not defined, we just return the value as str
                 try:
-                    result[i] = row[i].decode('utf-8')
+                    result[i] = row[i].decode("utf-8")
                 except UnicodeDecodeError:
                     result[i] = row[i]
             except (ValueError, TypeError) as err:
-                err.message = "{0} (field {1})".format(str(err), field[0])
+                err.message = f"{err} (field {field[0]})"
                 raise
 
             i += 1
 
         return tuple(result)
 
-    def _FLOAT_to_python(self, value, desc=None):  # pylint: disable=C0103
+    # pylint: disable=unused-argument
+    @staticmethod
+    def _float_to_python(value, desc=None):
         """
         Returns value as float type.
         """
         return float(value)
 
-    _DOUBLE_to_python = _FLOAT_to_python
+    _double_to_python = _float_to_python
 
-    def _INT_to_python(self, value, desc=None):  # pylint: disable=C0103
+    @staticmethod
+    def _int_to_python(value, desc=None):
         """
         Returns value as int type.
         """
         return int(value)
 
-    _TINY_to_python = _INT_to_python
-    _SHORT_to_python = _INT_to_python
-    _INT24_to_python = _INT_to_python
-    _LONG_to_python = _INT_to_python
-    _LONGLONG_to_python = _INT_to_python
+    _tiny_to_python = _int_to_python
+    _short_to_python = _int_to_python
+    _int24_to_python = _int_to_python
+    _long_to_python = _int_to_python
+    _longlong_to_python = _int_to_python
 
-    def _DECIMAL_to_python(self, value, desc=None):  # pylint: disable=C0103
+    def _decimal_to_python(self, value, desc=None):
         """
         Returns value as a decimal.Decimal.
         """
         val = value.decode(self.charset)
         return Decimal(val)
 
-    _NEWDECIMAL_to_python = _DECIMAL_to_python
+    _newdecimal_to_python = _decimal_to_python
 
-    def _str(self, value, desc=None):
+    @staticmethod
+    def _str(value, desc=None):
         """
         Returns value as str type.
         """
         return str(value)
 
-    def _BIT_to_python(self, value, dsc=None):  # pylint: disable=C0103
+    @staticmethod
+    def _bit_to_python(value, dsc=None):
         """Returns BIT columntype as integer"""
         int_val = value
         if len(int_val) < 8:
-            int_val = b'\x00' * (8 - len(int_val)) + int_val
-        return struct_unpack('>Q', int_val)[0]
+            int_val = b"\x00" * (8 - len(int_val)) + int_val
+        return struct.unpack(">Q", int_val)[0]
 
-    def _DATE_to_python(self, value, dsc=None):  # pylint: disable=C0103
+    @staticmethod
+    def _date_to_python(value, dsc=None):
         """Converts TIME column MySQL to a python datetime.datetime type.
 
         Raises ValueError if the value can not be converted.
@@ -473,22 +507,22 @@ class MySQLConverter(MySQLConverterBase):
         if isinstance(value, datetime.date):
             return value
         try:
-            parts = value.split(b'-')
+            parts = value.split(b"-")
             if len(parts) != 3:
-                raise ValueError("invalid datetime format: {} len: {}"
-                                 "".format(parts, len(parts)))
+                raise ValueError(f"invalid datetime format: {parts} len: {len(parts)}")
             try:
                 return datetime.date(int(parts[0]), int(parts[1]), int(parts[2]))
             except ValueError:
                 return None
         except (IndexError, ValueError):
             raise ValueError(
-                "Could not convert {0} to python datetime.timedelta".format(
-                    value))
+                f"Could not convert {value} to python datetime.timedelta"
+            ) from None
 
-    _NEWDATE_to_python = _DATE_to_python
+    _NEWDATE_to_python = _date_to_python
 
-    def _TIME_to_python(self, value, dsc=None):  # pylint: disable=C0103
+    @staticmethod
+    def _time_to_python(value, dsc=None):
         """Converts TIME column value to python datetime.time value type.
 
         Converts the TIME column MySQL type passed as bytes to a python
@@ -499,23 +533,26 @@ class MySQLConverter(MySQLConverterBase):
         Returns datetime.time type.
         """
         try:
-            (hms, mcs) = value.split(b'.')
-            mcs = int(mcs.ljust(6, b'0'))
+            (hms, mcs) = value.split(b".")
+            mcs = int(mcs.ljust(6, b"0"))
         except (TypeError, ValueError):
             hms = value
             mcs = 0
         try:
-            (hours, mins, secs) = [int(d) for d in hms.split(b':')]
-            if value[0] == 45 or value[0] == '-':  # if PY3 or PY2
+            (hours, mins, secs) = [int(d) for d in hms.split(b":")]
+            if value[0] == 45 or value[0] == "-":
                 mins, secs, mcs = -mins, -secs, -mcs
-            return datetime.timedelta(hours=hours, minutes=mins,
-                                      seconds=secs, microseconds=mcs)
+            return datetime.timedelta(
+                hours=hours, minutes=mins, seconds=secs, microseconds=mcs
+            )
         except (IndexError, TypeError, ValueError):
-            raise ValueError(CONVERT_ERROR.format(value=value,
-                                                  pytype="datetime.timedelta"))
+            raise ValueError(
+                CONVERT_ERROR.format(value=value, pytype="datetime.timedelta")
+            ) from None
 
-    def _DATETIME_to_python(self, value, dsc=None):  # pylint: disable=C0103
-        """"Converts DATETIME column value to python datetime.time value type.
+    @staticmethod
+    def _datetime_to_python(value, dsc=None):
+        """Converts DATETIME column value to python datetime.time value type.
 
         Converts the DATETIME column MySQL type passed as bytes to a python
         datetime.datetime type.
@@ -526,45 +563,50 @@ class MySQLConverter(MySQLConverterBase):
             return value
         datetime_val = None
         try:
-            (date_, time_) = value.split(b' ')
+            (date_, time_) = value.split(b" ")
             if len(time_) > 8:
-                (hms, mcs) = time_.split(b'.')
-                mcs = int(mcs.ljust(6, b'0'))
+                (hms, mcs) = time_.split(b".")
+                mcs = int(mcs.ljust(6, b"0"))
             else:
                 hms = time_
                 mcs = 0
-            dtval = [int(i) for i in date_.split(b'-')] + \
-                    [int(i) for i in hms.split(b':')] + [mcs, ]
+            dtval = (
+                [int(i) for i in date_.split(b"-")]
+                + [int(i) for i in hms.split(b":")]
+                + [
+                    mcs,
+                ]
+            )
             if len(dtval) < 6:
-                raise ValueError("invalid datetime format: {} len: {}"
-                                 "".format(dtval, len(dtval)))
-            else:
-                # Note that by default MySQL accepts invalid timestamps
-                # (this is also backward compatibility).
-                # Traditionaly C/py returns None for this well formed but
-                # invalid datetime for python like '0000-00-00 HH:MM:SS'.
-                try:
-                    datetime_val = datetime.datetime(*dtval)
-                except ValueError:
-                    return None
+                raise ValueError(f"invalid datetime format: {dtval} len: {len(dtval)}")
+            # Note that by default MySQL accepts invalid timestamps
+            # (this is also backward compatibility).
+            # Traditionaly C/py returns None for this well formed but
+            # invalid datetime for python like '0000-00-00 HH:MM:SS'.
+            try:
+                datetime_val = datetime.datetime(*dtval)
+            except ValueError:
+                return None
         except (IndexError, TypeError):
-            raise ValueError(CONVERT_ERROR.format(value=value,
-                                                  pytype="datetime.timedelta"))
+            raise ValueError(
+                CONVERT_ERROR.format(value=value, pytype="datetime.timedelta")
+            ) from None
 
         return datetime_val
 
-    _TIMESTAMP_to_python = _DATETIME_to_python
+    _timestamp_to_python = _datetime_to_python
 
-    def _YEAR_to_python(self, value, desc=None):  # pylint: disable=C0103
+    @staticmethod
+    def _year_to_python(value, desc=None):
         """Returns YEAR column type as integer"""
         try:
             year = int(value)
-        except ValueError:
-            raise ValueError("Failed converting YEAR to int (%s)" % value)
+        except ValueError as err:
+            raise ValueError(f"Failed converting YEAR to int ({value})") from err
 
         return year
 
-    def _SET_to_python(self, value, dsc=None):  # pylint: disable=C0103
+    def _set_to_python(self, value, dsc=None):
         """Returns SET column type as set
 
         Actually, MySQL protocol sees a SET as a string type field. So this
@@ -577,48 +619,46 @@ class MySQLConverter(MySQLConverterBase):
         if not val:
             return set()
         try:
-            set_type = set(val.split(','))
-        except ValueError:
-            raise ValueError("Could not convert set %s to a sequence." % value)
+            set_type = set(val.split(","))
+        except ValueError as err:
+            raise ValueError(f"Could not convert set {value} to a sequence") from err
         return set_type
 
-    def _STRING_to_python(self, value, dsc=None):  # pylint: disable=C0103
+    def _string_to_python(self, value, dsc=None):
         """
         Note that a SET is a string too, but using the FieldFlag we can see
         whether we have to split it.
 
         Returns string typed columns as string type.
         """
-        if dsc is not None:
-            # Check if we deal with a SET
-            if dsc[7] & FieldFlag.SET:
-                return self._SET_to_python(value, dsc)
-            if dsc[7] & FieldFlag.BINARY:
-                if self.charset != 'binary' and not isinstance(value, str):
-                    try:
-                        return value.decode(self.charset)
-                    except (LookupError, UnicodeDecodeError):
-                        return value
-                else:
-                    return value
-
-        if self.charset == 'binary':
+        if self.charset == "binary":
             return value
+        if dsc is not None:
+            if dsc[1] == FieldType.JSON and self.use_unicode:
+                return value.decode(self.charset)
+            if dsc[7] & FieldFlag.SET:
+                return self._set_to_python(value, dsc)
+            if dsc[8] == 63:  # 'binary' charset
+                return value
         if isinstance(value, (bytes, bytearray)) and self.use_unicode:
-            return value.decode(self.charset)
+            try:
+                return value.decode(self.charset)
+            except UnicodeDecodeError:
+                return value
 
         return value
 
-    _VAR_STRING_to_python = _STRING_to_python
-    _JSON_to_python = _STRING_to_python
+    _var_string_to_python = _string_to_python
+    _json_to_python = _string_to_python
 
-    def _BLOB_to_python(self, value, dsc=None):  # pylint: disable=C0103
+    def _blob_to_python(self, value, dsc=None):
         """Convert BLOB data type to Python."""
         if dsc is not None:
             if dsc[7] & FieldFlag.BLOB and dsc[7] & FieldFlag.BINARY:
                 return bytes(value)
-        return self._STRING_to_python(value, dsc)
+        return self._string_to_python(value, dsc)
 
-    _LONG_BLOB_to_python = _BLOB_to_python
-    _MEDIUM_BLOB_to_python = _BLOB_to_python
-    _TINY_BLOB_to_python = _BLOB_to_python
+    _long_blob_to_python = _blob_to_python
+    _medium_blob_to_python = _blob_to_python
+    _tiny_blob_to_python = _blob_to_python
+    # pylint: enable=unused-argument
