@@ -1,4 +1,4 @@
-# Copyright (c) 2016, 2020, Oracle and/or its affiliates.
+# Copyright (c) 2016, 2022, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -35,8 +35,9 @@ from .locales import get_client_error
 
 class Error(Exception):
     """Exception that is base class for all other error exceptions."""
+
     def __init__(self, msg=None, errno=None, values=None, sqlstate=None):
-        super(Error, self).__init__()
+        super().__init__()
         self.msg = msg
         self._full_msg = self.msg
         self.errno = errno or -1
@@ -48,15 +49,12 @@ class Error(Exception):
                 try:
                     self.msg = self.msg % values
                 except TypeError as err:
-                    self.msg = "{0} (Warning: {1})".format(self.msg, str(err))
+                    self.msg = f"{self.msg} (Warning: {err})"
         elif not self.msg:
             self._full_msg = self.msg = "Unknown error"
 
         if self.msg and self.errno != -1:
-            fields = {
-                "errno": self.errno,
-                "msg": self.msg
-            }
+            fields = {"errno": self.errno, "msg": self.msg}
             if self.sqlstate:
                 fmt = "{errno} ({state}): {msg}"
                 fields["state"] = self.sqlstate
@@ -72,69 +70,56 @@ class Error(Exception):
 
 class InterfaceError(Error):
     """Exception for errors related to the interface."""
-    pass
 
 
 class DatabaseError(Error):
     """Exception for errors related to the database."""
-    pass
 
 
 class InternalError(DatabaseError):
     """Exception for errors internal database errors."""
-    pass
 
 
 class OperationalError(DatabaseError):
     """Exception for errors related to the database's operation."""
-    pass
 
 
 class ProgrammingError(DatabaseError):
     """Exception for errors programming errors."""
-    pass
 
 
 class IntegrityError(DatabaseError):
     """Exception for errors regarding relational integrity."""
-    pass
 
 
 class DataError(DatabaseError):
     """Exception for errors reporting problems with processed data."""
-    pass
 
 
 class NotSupportedError(DatabaseError):
     """Exception for errors when an unsupported database feature was used."""
-    pass
 
 
 class PoolError(Error):
     """Exception for errors relating to connection pooling."""
-    pass
 
-# pylint: disable=W0622
-class TimeoutError(Error):
+
+class TimeoutError(Error):  # pylint: disable=redefined-builtin
     """Exception for errors relating to connection timeout."""
-    pass
 
 
 def intread(buf):
     """Unpacks the given buffer to an integer."""
-    try:
-        if isinstance(buf, int):
-            return buf
-        length = len(buf)
-        if length == 1:
-            return buf[0]
-        elif length <= 4:
-            tmp = buf + b"\x00" * (4 - length)
-            return struct_unpack("<I", tmp)[0]
-        tmp = buf + b"\x00" * (8 - length)
-        return struct_unpack("<Q", tmp)[0]
-    except:
-        raise
+    if isinstance(buf, int):
+        return buf
+    length = len(buf)
+    if length == 1:
+        return buf[0]
+    if length <= 4:
+        tmp = buf + b"\x00" * (4 - length)
+        return struct_unpack("<I", tmp)[0]
+    tmp = buf + b"\x00" * (8 - length)
+    return struct_unpack("<Q", tmp)[0]
 
 
 def read_int(buf, size):
@@ -142,12 +127,7 @@ def read_int(buf, size):
 
     Returns a tuple (truncated buffer, int).
     """
-
-    try:
-        res = intread(buf[0:size])
-    except:
-        raise
-
+    res = intread(buf[0:size])
     return (buf[size:], res)
 
 
@@ -169,8 +149,7 @@ def get_mysql_exception(errno, msg=None, sqlstate=None):
     Returns an Exception.
     """
     try:
-        return _ERROR_EXCEPTIONS[errno](msg=msg, errno=errno,
-                                        sqlstate=sqlstate)
+        return _ERROR_EXCEPTIONS[errno](msg=msg, errno=errno, sqlstate=sqlstate)
     except KeyError:
         # Error was not mapped to particular exception
         pass
@@ -179,8 +158,9 @@ def get_mysql_exception(errno, msg=None, sqlstate=None):
         return DatabaseError(msg=msg, errno=errno)
 
     try:
-        return _SQLSTATE_CLASS_EXCEPTION[sqlstate[0:2]](msg=msg, errno=errno,
-                                                        sqlstate=sqlstate)
+        return _SQLSTATE_CLASS_EXCEPTION[sqlstate[0:2]](
+            msg=msg, errno=errno, sqlstate=sqlstate
+        )
     except KeyError:
         # Return default InterfaceError
         return DatabaseError(msg=msg, errno=errno, sqlstate=sqlstate)
@@ -200,8 +180,7 @@ def get_exception(packet):
         if packet[4] != 255:
             raise ValueError("Packet is not an error packet")
     except IndexError as err:
-        return InterfaceError("Failed getting Error information ({0})"
-                              "".format(err))
+        return InterfaceError(f"Failed getting Error information ({err})")
 
     sqlstate = None
     try:
@@ -209,17 +188,17 @@ def get_exception(packet):
         packet, errno = read_int(packet, 2)
         if packet[0] != 35:
             # Error without SQLState
-            if isinstance(packet, (bytes, bytearray)):
-                errmsg = packet.decode("utf8")
-            else:
-                errmsg = packet
+            errmsg = (
+                packet.decode("utf8")
+                if isinstance(packet, (bytes, bytearray))
+                else packet
+            )
         else:
             packet, sqlstate = read_bytes(packet[1:], 5)
             sqlstate = sqlstate.decode("utf8")
             errmsg = packet.decode("utf8")
-    except Exception as err:  # pylint: disable=W0703
-        return InterfaceError("Failed getting Error information ({0})"
-                              "".format(err))
+    except (IndexError, ValueError) as err:
+        return InterfaceError(f"Failed getting Error information ({err})")
     else:
         return get_mysql_exception(errno, errmsg, sqlstate)
 
@@ -251,7 +230,7 @@ _SQLSTATE_CLASS_EXCEPTION = {
     "3F": ProgrammingError,  # invalid schema name
     "40": InternalError,  # transaction rollback
     "42": ProgrammingError,  # syntax error or access rule violation
-    "44": InternalError,   # with check option violation
+    "44": InternalError,  # with check option violation
     "HZ": OperationalError,  # remote database access
     "XA": IntegrityError,
     "0K": OperationalError,
