@@ -1,4 +1,4 @@
-# Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2016, 2022, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
@@ -34,50 +34,40 @@ import sys
 
 from datetime import datetime, timedelta
 
-from .dbdoc import DbDoc
 from .charsets import MYSQL_CHARACTER_SETS
-from .compat import STRING_TYPES
+from .dbdoc import DbDoc
 from .helpers import decode_from_bytes, deprecated
 
 
-# pylint: disable=C0111
-def from_protobuf(col_type, payload):
+# pylint: disable=missing-class-docstring,missing-function-docstring
+def from_protobuf(column, payload):
     if len(payload) == 0:
         return None
 
+    if column.get_type() == ColumnType.STRING:
+        return decode_from_bytes(payload[:-1])  # Strip trailing char
+
     try:
-        return {
-            ColumnProtoType.SINT: varsint_from_protobuf,
-            ColumnProtoType.UINT: varint_from_protobuf,
-            ColumnProtoType.BYTES: bytes_from_protobuf,
-            ColumnProtoType.DATETIME: datetime_from_protobuf,
-            ColumnProtoType.TIME: time_from_protobuf,
-            ColumnProtoType.FLOAT: float_from_protobuf,
-            ColumnProtoType.DOUBLE: double_from_protobuf,
-            ColumnProtoType.BIT: varint_from_protobuf,
-            ColumnProtoType.SET: set_from_protobuf,
-            ColumnProtoType.ENUM: bytes_from_protobuf,
-            ColumnProtoType.DECIMAL: decimal_from_protobuf,
-        }[col_type](payload)
+        return ColumnProtoType.converter_map[column.get_proto_type()](payload)
     except KeyError as err:
-        sys.stderr.write("{0}".format(err))
-        sys.stderr.write("{0}".format(payload.encode("hex")))
+        sys.stderr.write(f"{err}")
+        sys.stderr.write(f"{payload.encode('hex')}")
         return None
 
 
 def bytes_from_protobuf(payload):
     # Strip trailing char
-    return decode_from_bytes(payload[:-1])
+    return payload[:-1]
 
 
 def float_from_protobuf(payload):
     assert len(payload) == 4
-    return struct.unpack("<f", payload)
+    return struct.unpack("<f", payload)[0]
 
 
 def double_from_protobuf(payload):
     assert len(payload) == 8
-    return struct.unpack("<d", payload)
+    return struct.unpack("<d", payload)[0]
 
 
 def varint_from_protobuf_stream(payload):
@@ -91,11 +81,11 @@ def varint_from_protobuf_stream(payload):
     for item in payload:
         char = item if isinstance(item, int) else ord(item)
         eos = (char & 0x80) == 0
-        cur_bits = (char & 0x7f)
+        cur_bits = char & 0x7F
         cur_bits <<= shift
         i |= cur_bits
         if eos:
-            return i, payload[cur + 1:]
+            return i, payload[cur + 1 :]
         cur += 1
         shift += 7
 
@@ -118,10 +108,10 @@ def varsint_from_protobuf(payload):
     # Zigzag encoded, revert it
     if i & 0x1:
         i = ~i
-        i = (i >> 1)
+        i = i >> 1
         i |= 1 << 63
     else:
-        i = (i >> 1)
+        i = i >> 1
 
     return i
 
@@ -155,30 +145,30 @@ def decimal_from_protobuf(payload):
 
     for item in payload:
         char = item if isinstance(item, int) else ord(item)
-        high_bcd = (char & 0xf0) >> 4
-        low_bcd = char & 0x0f
-        if high_bcd < 0x0a:
+        high_bcd = (char & 0xF0) >> 4
+        low_bcd = char & 0x0F
+        if high_bcd < 0x0A:
             digits.append(high_bcd)
-            if low_bcd < 0x0a:
+            if low_bcd < 0x0A:
                 digits.append(low_bcd)
-            elif low_bcd == 0x0c:
+            elif low_bcd == 0x0C:
                 sign = 0
                 break
-            elif low_bcd == 0x0d:
+            elif low_bcd == 0x0D:
                 sign = 1
                 break
             else:
                 raise ValueError("Invalid BCD")
-        elif high_bcd == 0x0c:
+        elif high_bcd == 0x0C:
             sign = 0
             assert low_bcd == 0x00
             break
-        elif high_bcd == 0x0d:
+        elif high_bcd == 0x0D:
             sign = 1
             assert low_bcd == 0x00
             break
         else:
-            raise ValueError("Invalid BCD: {0}".format(high_bcd))
+            raise ValueError(f"Invalid BCD: {high_bcd}")
 
     return decimal.Decimal((sign, digits, -scale))
 
@@ -232,15 +222,16 @@ def time_from_protobuf(payload):
         elif useconds:
             useconds *= -1
 
-    return timedelta(hours=hour, minutes=minutes, seconds=seconds,
-                     microseconds=useconds)
+    return timedelta(
+        hours=hour, minutes=minutes, seconds=seconds, microseconds=useconds
+    )
 
 
-class Collations(object):
+class Collations:
     UTF8_GENERAL_CI = 33
 
 
-class ColumnType(object):
+class ColumnType:
     BIT = 1
     TINYINT = 2
     SMALLINT = 3
@@ -290,33 +281,57 @@ class ColumnType(object):
 
     @classmethod
     def is_char(cls, col_type):
-        return col_type in (cls.CHAR, cls.VARCHAR,)
+        return col_type in (
+            cls.CHAR,
+            cls.VARCHAR,
+        )
 
     @classmethod
     def is_binary(cls, col_type):
-        return col_type in (cls.BINARY, cls.VARBINARY,)
+        return col_type in (
+            cls.BINARY,
+            cls.VARBINARY,
+        )
 
     @classmethod
     def is_text(cls, col_type):
-        return col_type in (cls.TEXT, cls.TINYTEXT, cls.MEDIUMTEXT,
-                            cls.LONGTEXT,)
+        return col_type in (
+            cls.TEXT,
+            cls.TINYTEXT,
+            cls.MEDIUMTEXT,
+            cls.LONGTEXT,
+        )
 
     @classmethod
     def is_decimals(cls, col_type):
-        return col_type in (cls.REAL, cls.DOUBLE, cls.FLOAT, cls.DECIMAL,
-                            cls.NUMERIC,)
+        return col_type in (
+            cls.REAL,
+            cls.DOUBLE,
+            cls.FLOAT,
+            cls.DECIMAL,
+            cls.NUMERIC,
+        )
 
     @classmethod
     def is_numeric(cls, col_type):
-        return col_type in (cls.BIT, cls.TINYINT, cls.SMALLINT, cls.MEDIUMINT,
-                            cls.INT, cls.BIGINT,)
+        return col_type in (
+            cls.BIT,
+            cls.TINYINT,
+            cls.SMALLINT,
+            cls.MEDIUMINT,
+            cls.INT,
+            cls.BIGINT,
+        )
 
     @classmethod
     def is_finite_set(cls, col_type):
-        return col_type in (cls.SET, cls.ENUM,)
+        return col_type in (
+            cls.SET,
+            cls.ENUM,
+        )
 
 
-class ColumnProtoType(object):
+class ColumnProtoType:
     SINT = 1
     UINT = 2
     DOUBLE = 5
@@ -329,8 +344,22 @@ class ColumnProtoType(object):
     BIT = 17
     DECIMAL = 18
 
+    converter_map = {
+        SINT: varsint_from_protobuf,
+        UINT: varint_from_protobuf,
+        BYTES: bytes_from_protobuf,
+        DATETIME: datetime_from_protobuf,
+        TIME: time_from_protobuf,
+        FLOAT: float_from_protobuf,
+        DOUBLE: double_from_protobuf,
+        BIT: varint_from_protobuf,
+        SET: set_from_protobuf,
+        ENUM: bytes_from_protobuf,
+        DECIMAL: decimal_from_protobuf,
+    }
 
-class Flags(object):
+
+class Flags:
     def __init__(self, value):
         self._allowed_flags = {}
         self._flag_names = {}
@@ -355,8 +384,8 @@ class Flags(object):
                 try:
                     flag_names.append(self._flag_names[flag])
                 except KeyError:
-                    sys.stderr.write("{0}".format(self._flag_names))
-                    sys.stderr.write("{0}".format(self.__class__.__dict__))
+                    sys.stderr.write(f"{self._flag_names}")
+                    sys.stderr.write(f"{self.__class__.__dict__}")
 
         return ",".join(flag_names)
 
@@ -401,10 +430,12 @@ class BytesContentType(ColumnFlags):
     GEOMETRY = 0x0001
     JSON = 0x0002
     XML = 0x0003
-# pylint: enable=C0111
 
 
-class Column(object):
+# pylint: enable=missing-class-docstring,missing-function-docstring
+
+
+class Column:
     """Represents meta data for a table column.
 
     Args:
@@ -423,10 +454,22 @@ class Column(object):
 
     .. versionchanged:: 8.0.12
     """
-    def __init__(self, col_type, catalog=None, schema=None, table=None,
-                 original_table=None, name=None, original_name=None,
-                 length=None, collation=None, fractional_digits=None,
-                 flags=None, content_type=None):
+
+    def __init__(
+        self,
+        col_type,
+        catalog=None,
+        schema=None,
+        table=None,
+        original_table=None,
+        name=None,
+        original_name=None,
+        length=None,
+        collation=None,
+        fractional_digits=None,
+        flags=None,
+        content_type=None,
+    ):
         self._schema = decode_from_bytes(schema)
         self._name = decode_from_bytes(name)
         self._original_name = decode_from_bytes(original_name)
@@ -450,25 +493,31 @@ class Column(object):
 
         if self._collation > 0:
             if self._collation >= len(MYSQL_CHARACTER_SETS):
-                raise ValueError("No mapping found for collation {0}"
-                                 "".format(self._collation))
+                raise ValueError(f"No mapping found for collation {self._collation}")
             info = MYSQL_CHARACTER_SETS[self._collation]
             self._character_set_name = info[0]
             self._collation_name = info[1]
-            self._is_binary = ("binary" in self._collation_name or
-                               "_bin" in self._collation_name)
+            self._is_binary = (
+                "binary" in self._collation_name or "_bin" in self._collation_name
+            )
         self._map_type()
         self._is_bytes = self._col_type in (
-            ColumnType.GEOMETRY, ColumnType.JSON, ColumnType.XML,
-            ColumnType.BYTES, ColumnType.STRING)
+            ColumnType.GEOMETRY,
+            ColumnType.JSON,
+            ColumnType.XML,
+            ColumnType.BYTES,
+            ColumnType.STRING,
+        )
 
     def __str__(self):
-        return str({
-            "col_type": self._col_type,
-            "schema": self._schema,
-            "table": self._table,
-            "flags": str(self._flags),
-        })
+        return str(
+            {
+                "col_type": self._col_type,
+                "schema": self._schema,
+                "table": self._table,
+                "flags": str(self._flags),
+            }
+        )
 
     def _map_bytes(self):
         """Map bytes."""
@@ -488,10 +537,10 @@ class Column(object):
         """Map datetime."""
         if self._length == 10:
             self._col_type = ColumnType.DATE
-        elif self._length == 19:
-            self._col_type = ColumnType.DATETIME
         elif self._flags & DatetimeColumnFlags.TIMESTAMP > 0:
             self._col_type = ColumnType.TIMESTAMP
+        elif self._length >= 19:
+            self._col_type = ColumnType.DATETIME
         else:
             raise ValueError("Datetime mapping scenario unhandled")
 
@@ -531,16 +580,13 @@ class Column(object):
             self._map_uint_type()
         elif self._proto_type == ColumnProtoType.FLOAT:
             self._col_type = ColumnType.FLOAT
-            self._is_number_signed = \
-                (self._flags & FloatColumnFlags.UNSIGNED) == 0
+            self._is_number_signed = (self._flags & FloatColumnFlags.UNSIGNED) == 0
         elif self._proto_type == ColumnProtoType.DECIMAL:
             self._col_type = ColumnType.DECIMAL
-            self._is_number_signed = \
-                (self._flags & FloatColumnFlags.UNSIGNED) == 0
+            self._is_number_signed = (self._flags & FloatColumnFlags.UNSIGNED) == 0
         elif self._proto_type == ColumnProtoType.DOUBLE:
             self._col_type = ColumnType.DOUBLE
-            self._is_number_signed = \
-                (self._flags & FloatColumnFlags.UNSIGNED) == 0
+            self._is_number_signed = (self._flags & FloatColumnFlags.UNSIGNED) == 0
         elif self._proto_type == ColumnProtoType.BYTES:
             self._map_bytes()
         elif self._proto_type == ColumnProtoType.TIME:
@@ -554,7 +600,7 @@ class Column(object):
         elif self._proto_type == ColumnProtoType.BIT:
             self._col_type = ColumnType.BIT
         else:
-            raise ValueError("Unknown column type {0}".format(self._proto_type))
+            raise ValueError(f"Unknown column type {self._proto_type}")
 
     @property
     def schema_name(self):
@@ -749,27 +795,30 @@ class Column(object):
         return self._is_bytes
 
 
-class Row(object):
+class Row:
     """Represents a row element returned from a SELECT query.
 
     Args:
-        rs (mysqlx.SqlResult or mysqlx.RowResult): The result set.
+        resultset (mysqlx.SqlResult or mysqlx.RowResult): The result set.
         fields (`list`): The list of fields.
     """
-    def __init__(self, rs, fields):
+
+    def __init__(self, resultset, fields):
         self._fields = fields
-        self._resultset = rs
+        self._resultset = resultset
+
+    def __repr__(self):
+        return repr(self._fields)
 
     def __getitem__(self, index):
         """Returns the value of a column by name or index.
 
         .. versionchanged:: 8.0.12
         """
-        int_index = self._resultset.index_of(index) \
-            if isinstance(index, STRING_TYPES) else index
-        if int_index == -1 and isinstance(index, STRING_TYPES):
-            raise ValueError("Column name '{0}' not found".format(index))
-        elif int_index >= len(self._fields) or int_index < 0:
+        int_index = self._resultset.index_of(index) if isinstance(index, str) else index
+        if int_index == -1 and isinstance(index, str):
+            raise ValueError(f"Column name '{index}' not found")
+        if int_index >= len(self._fields) or int_index < 0:
             raise IndexError("Index out of range")
         return self._fields[int_index]
 
@@ -786,16 +835,17 @@ class Row(object):
         if int_index >= len(self._fields):
             raise IndexError("Argument out of range")
         if int_index == -1:
-            raise ValueError("Column name '{0}' not found".format(str_index))
+            raise ValueError(f"Column name '{str_index}' not found")
         return str(self._fields[int_index])
 
 
-class BaseResult(object):
+class BaseResult:
     """Provides base functionality for result objects.
 
     Args:
         connection (mysqlx.connection.Connection): The Connection object.
     """
+
     def __init__(self, connection):
         self._connection = connection
         self._closed = False
@@ -835,8 +885,7 @@ class BaseResult(object):
         return len(self._warnings)
 
     def set_closed(self, flag):
-        """Sets if resultset fetch is done.
-        """
+        """Sets if resultset fetch is done."""
         self._closed = flag
 
     def append_warning(self, level, code, msg):
@@ -850,18 +899,15 @@ class BaseResult(object):
         self._warnings.append({"level": level, "code": code, "msg": msg})
 
     def set_generated_ids(self, generated_ids):
-        """Sets the generated ids.
-        """
+        """Sets the generated ids."""
         self._generated_ids = generated_ids
 
     def set_generated_insert_id(self, generated_id):
-        """Sets the generated insert id.
-        """
+        """Sets the generated insert id."""
         self._generated_id = generated_id
 
     def set_rows_affected(self, total):
-        """Sets the number of rows affected.
-        """
+        """Sets the number of rows affected."""
         self._rows_affected = total
 
 
@@ -873,8 +919,9 @@ class Result(BaseResult):
         connection (mysqlx.connection.Connection): The Connection object.
                                                    ids (`list`): A list of IDs.
     """
+
     def __init__(self, connection=None, ids=None):
-        super(Result, self).__init__(connection)
+        super().__init__(connection)
         self._ids = ids
 
         if connection is not None:
@@ -907,8 +954,7 @@ class Result(BaseResult):
         return self._generated_id
 
     def get_generated_ids(self):
-        """Returns the generated ids.
-        """
+        """Returns the generated ids."""
         return self._generated_ids
 
 
@@ -919,8 +965,9 @@ class BufferingResult(BaseResult):
         connection (mysqlx.connection.Connection): The Connection object.
                                                    ids (`list`): A list of IDs.
     """
+
     def __init__(self, connection):
-        super(BufferingResult, self).__init__(connection)
+        super().__init__(connection)
         self._columns = []
         self._has_data = False
         self._has_more_results = False
@@ -940,12 +987,11 @@ class BufferingResult(BaseResult):
     def _init_result(self):
         """Initialize the result."""
         self._columns = self._connection.get_column_metadata(self)
-        self._has_more_data = True if len(self._columns) > 0 else False
+        self._has_more_data = len(self._columns) > 0
         self._items = []
         self._page_size = 20
         self._position = -1
-        self._connection.set_active_result(self if self._has_more_data
-                                           else None)
+        self._connection.set_active_result(self if self._has_more_data else None)
 
     def _read_item(self, dumping):
         """Read item.
@@ -962,9 +1008,8 @@ class BufferingResult(BaseResult):
         item = [None] * len(row["field"])
         if not dumping:
             for key in range(len(row["field"])):
-                col = self._columns[key]
-                item[key] = from_protobuf(col.get_proto_type(),
-                                          row["field"][key])
+                column = self._columns[key]
+                item[key] = from_protobuf(column, row["field"][key])
         return Row(self, item)
 
     def _page_in_items(self):
@@ -1044,8 +1089,6 @@ class RowResult(BufferingResult):
     Args:
         connection (mysqlx.connection.Connection): The Connection object.
     """
-    def __init__(self, connection):
-        super(RowResult, self).__init__(connection)
 
     @property
     def columns(self):
@@ -1069,8 +1112,6 @@ class SqlResult(RowResult):
     Args:
         connection (mysqlx.connection.Connection): The Connection object.
     """
-    def __init__(self, connection):
-        super(SqlResult, self).__init__(connection)
 
     def get_autoincrement_value(self):
         """Returns the identifier for the last record inserted.
@@ -1102,6 +1143,7 @@ class SqlResult(RowResult):
         """
         return self._has_data
 
+
 class DocResult(BufferingResult):
     """Allows traversing the DbDoc objects returned by a Collection.find
     operation.
@@ -1109,8 +1151,6 @@ class DocResult(BufferingResult):
     Args:
         connection (mysqlx.connection.Connection): The Connection object.
     """
-    def __init__(self, connection):
-        super(DocResult, self).__init__(connection)
 
     def _read_item(self, dumping):
         """Read item.
@@ -1121,7 +1161,7 @@ class DocResult(BufferingResult):
         Returns:
             :class:`mysqlx.DbDoc`: A `DbDoc` object.
         """
-        row = super(DocResult, self)._read_item(dumping)
+        row = super()._read_item(dumping)
         if row is None:
             return None
         return DbDoc(decode_from_bytes(row[0]))
